import json
import os

from aiohttp import ClientSession

from .google_patent_common import parseGooglePatentInfoHtml, GooglePatentInfo
from .utils import createDirs, downloadFile

async def getHtmlAsync(url,proxies=None,headers=None):
    if proxies:
        if proxies=="clash":
            proxies = 'http://127.0.0.1:7890'
    async with ClientSession() as session:
        async with session.get(url,proxy=proxies,headers=headers) as response:
            return await response.text(encoding="utf-8")

async def getGooglePatentInfoByUrlAsync(url,proxies=None,headers=None):
    html=await getHtmlAsync(url,proxies,headers)
    return parseGooglePatentInfoHtml(html,url)
async def dictToGooglePatentInfoAsync(d):
    return GooglePatentInfo(**d)
async def getGooglePatentInfoAsync(query_txt,base_url="https://patents.google.com",proxies=None,headers=None,language="auto"):
    if language == "auto":
        if query_txt.find("CN")!=-1:
            language = "zh"
        else:
            language = "en"
    url=f"{base_url}/patent/{query_txt}/{language}?oq={query_txt}"
    d=await getGooglePatentInfoByUrlAsync(url,proxies=proxies,headers=headers)
    if d is None:
        return None
    return await dictToGooglePatentInfoAsync(d)
async def downloadGooglePdfAsync(pdf_url,save_path):
    return downloadFile(pdf_url,save_path)
async def autoGoogleSpiderAsync(query_txt,save_dir="data",save_pdf=True,base_url="https://patents.google.com",proxies=None,headers=None):
    google_patent_info=await getGooglePatentInfoAsync(query_txt,base_url=base_url,proxies=proxies,headers=headers)
    if google_patent_info is None:return False
    createDirs(save_dir)
    base_patent_dir = os.path.join(save_dir, google_patent_info.publication_number)
    createDirs(base_patent_dir)
    patent_info_dataframe,patent_content_dataframe=google_patent_info.toDataFrame()

    patent_info_file = os.path.join(base_patent_dir, f"info.xlsx")
    patent_info_dataframe.to_excel(patent_info_file,index=False)

    patent_content_file = os.path.join(base_patent_dir, f"content.xlsx")
    patent_content_dataframe.to_excel(patent_content_file,index=False)
    with open(os.path.join(base_patent_dir, f"origin_data.json"), "w", encoding="utf-8") as f:
        json.dump(google_patent_info.__dict__, f, ensure_ascii=False)
    if save_pdf:
        patent_pdf_file = os.path.join(base_patent_dir, f"patent.pdf")
        await downloadGooglePdfAsync(google_patent_info.pdf_url, patent_pdf_file)
    return True



import json
import os

import requests

from .utils import createDirs, downloadFile
from .google_patent_common import parseGooglePatentInfoHtml, GooglePatentInfo

def getHtml(url,proxies=None,headers=None):
    if proxies:
        if proxies=="clash":
            proxies = {
                'http': 'http://127.0.0.1:7890',
                'https': 'http://127.0.0.1:7890',
            }
    res=requests.get(url,proxies=proxies,headers=headers)
    res.encoding=res.apparent_encoding
    return res.text
def getGooglePatentInfoByUrl(url,proxies=None,headers=None,engine="lxml"):
    html=getHtml(url,proxies=proxies,headers=headers)
    return parseGooglePatentInfoHtml(html,url,engine)

def getGooglePatentInfo(query_txt,base_url="https://patents.google.com",proxies=None,headers=None,language="auto",engine="lxml"):
    if language == "auto":
        if query_txt.find("CN")!=-1:
            language = "zh"
        else:
            language = "en"
    url=f"{base_url}/patent/{query_txt}/{language}?oq={query_txt}"
    d=getGooglePatentInfoByUrl(url,proxies=proxies,headers=headers,engine=engine)
    if d is None:
        return None
    return GooglePatentInfo(**d)
def downloadGooglePdf(pdf_url,save_path):
    return downloadFile(pdf_url,save_path)
def autoGoogleSpider(query_txt,save_dir="data",save_pdf=True,base_url="https://patents.google.com",proxies=None,headers=None):
    google_patent_info=getGooglePatentInfo(query_txt,base_url=base_url,proxies=proxies,headers=headers)
    if google_patent_info is None:return False
    createDirs(save_dir)
    base_patent_dir = os.path.join(save_dir, google_patent_info.publication_number)
    createDirs(base_patent_dir)
    patent_info_dataframe,patent_content_dataframe=google_patent_info.toDataFrame()

    patent_info_file = os.path.join(base_patent_dir, f"info.xlsx")
    patent_info_dataframe.to_excel(patent_info_file,index=False)

    patent_content_file = os.path.join(base_patent_dir, f"content.xlsx")
    patent_content_dataframe.to_excel(patent_content_file,index=False)
    with open(os.path.join(base_patent_dir, f"origin_data.json"), "w", encoding="utf-8") as f:
        json.dump(google_patent_info.__dict__, f, ensure_ascii=False)
    if save_pdf:
        patent_pdf_file = os.path.join(base_patent_dir, f"patent.pdf")
        downloadGooglePdf(google_patent_info.pdf_url, patent_pdf_file)
    return True



from .fpo_patent import *
from .google_patent import *
from .utils import createDirs,log
from .google_patent_async import *
from .google_patent_common import *
__all__ = [
    'downloadFile',
    'createDirs',
    'getFpoPatentInfo',
    "getFpoPatentInfoByUrl",
    "getFpoSearchResult",
    "downloadFpoPdf",
    "downloadFpoPdfByUrl",
    "autoFpoSpider",
    "getGooglePatentInfo",
    "getGooglePatentInfoByUrl",
    "getGooglePatentInfoAsync",
    "getGooglePatentInfoByUrlAsync",
    "downloadGooglePdfAsync",
    "getHtmlAsync",
    "parseGooglePatentInfoHtml"
]


import json
import os.path

import pandas as pd
import requests
from bs4 import BeautifulSoup
from .utils import createDirs, downloadFile


class FpoSearchResult:
    def __init__(self, url,label,patent_id, patent_title, patent_abstract,patent_total_num,page):
        self.label=label
        self.patent_id = patent_id
        self.patent_title = patent_title
        self.patent_abstract = patent_abstract
        self.url = url
        self.patent_total_num=patent_total_num
        self.page=page

class FpoPatentInfo:
    def __init__(self,url,patent_id,title,abstract,inventors,application_number,publication_date,filing_date,assignee,primary_class,other_classes,international_classes,field_of_search,pdf_url,view_patent_images,us_patent_references,other_references,primary_examiner,attorney_agent_or_firm,claims,description):
        self.url=url
        self.patent_id=patent_id
        self.title=title
        self.abstract=abstract
        self.inventors=inventors
        self.application_number=application_number
        self.publication_date=publication_date
        self.filing_date=filing_date
        self.assignee=assignee
        self.primary_class=primary_class
        self.other_classes=other_classes
        self.international_classes=international_classes
        self.field_of_search=field_of_search
        self.pdf_url=pdf_url
        self.us_patent_references=us_patent_references
        self.other_references=other_references
        self.primary_examiner=primary_examiner
        self.attorney_agent_or_firm=attorney_agent_or_firm
        self.claims=claims
        self.description=description
        self.view_patent_images=view_patent_images
    def toDataFrame(self):
        x={
            'patent_id':[self.patent_id],
            'title':[self.title],
            'abstract':[self.abstract],
            'inventors':[self.inventors],
            'application_number':[self.application_number],
            'publication_date':[self.publication_date],
            'filing_date':[self.filing_date],
            'assignee':[self.assignee],
            'primary_class':[self.primary_class],
            'other_classes':[self.other_classes],
            'international_classes':[self.international_classes],
            'field_of_search':[self.field_of_search],
            'pdf_url':[self.pdf_url],
            'us_patent_references':[self.us_patent_references],
            'other_references':[self.other_references],
            'primary_examiner':[self.primary_examiner],
            'attorney_agent_or_firm':[self.attorney_agent_or_firm],
            'claims':[self.claims],
        }
        x1={

        }
        for k,v in self.description.items():
            x1[k]=[v]
        return pd.DataFrame(x),pd.DataFrame(x1)
def getFpoSearchResult(query_txt,page=1,sort="relevance",srch="top",patents_us="on",patents_other="off"):
    url=f"https://www.freepatentsonline.com/result.html?p={page}&sort={sort}&srch={srch}&query_txt={query_txt}&submit=&patents_us={patents_us}&patents_other={patents_other}"
    res=requests.get(url)
    soup = BeautifulSoup(res.text, "lxml")
    if page>200:
        raise Exception("无法浏览超过200页的数据")
    #获取专利总量
    patent_total_num=soup.select_one("#hits")["value"]
    t_table=soup.select_one("#results > div.legacy-container > div > div > table")
    tr_list=t_table.find_all("tr")[1:]
    res=[]
    for tr in tr_list:
        td_list=tr.find_all("td")
        label=int(td_list[0].get_text())
        id = td_list[1].get_text().strip()
        url=f"https://www.freepatentsonline.com"+td_list[2].find("a")["href"]
        title=td_list[2].find("a").get_text().strip()
        abstract=td_list[2].get_text().split("\n")[-1].strip()
        res.append(FpoSearchResult(url,label,id,title,abstract,patent_total_num,page))
    return res

def getFpoPatentInfoByUrl(url):
    res=requests.get(url)
    soup = BeautifulSoup(res.text, "lxml")
    wrapper=soup.select_one("body > div > div > div.fixed-width.document-details-wrapper")
    div_list=wrapper.find_all("div",{"class":"disp_doc2"})
    field_dict={'title': '', 'abstract': '', 'inventors': '', 'application_number': '', 'publication_date': '', 'filing_date': '',  'assignee': '', 'primary_class': '', 'other_classes': '', 'international_classes': '', 'field_of_search': '', 'view_patent_images': '', 'us_patent_references':'','other_references': '', 'primary_examiner': '', 'attorney_agent_or_firm': '', 'claims': '', 'description': ''}
    for div in div_list:
        div_title=div.find("div",{"class":"disp_elm_title"})
        div_text=div.find("div",{"class":"disp_elm_text"})
        if div_title:
            title = div_title.get_text().replace(" ", "_").lower().replace(",", "").replace(":", "")
            text=div_text.get_text().strip()
            if title not in field_dict.keys():continue
            if title=="inventors":
                text=text.replace("\n",";").replace(" ","")
            if title=="other_classes":
                text=text.replace("\n","").replace(" ","")
            if title=="view_patent_images":
                field_dict["pdf_url"]="https://www.freepatentsonline.com"+div_text.find("a")["href"]
            if title=="other_references":
                text_list=text.split("\n")
                for index,t in enumerate(text_list):
                    text_list[index]=t.strip()
                text="|".join(text_list)
            if title=="us_patent_references":
                tr_list=div_text.find_all("tr")
                res_list=[]
                for tr in tr_list:
                    us_patent_references_dict={}
                    td_list=tr.find_all("td")
                    us_patent_references_dict["patent_id"]=td_list[0].get_text().strip()
                    us_patent_references_dict["title"]=td_list[1].get_text().strip()
                    us_patent_references_dict["date"]=td_list[2].get_text().strip()
                    us_patent_references_dict["author"]=td_list[3].get_text().strip()
                    res_list.append(us_patent_references_dict)
                text=res_list
            if title=="description":
                content_dict={
                }
                headings = soup.find_all("h1")
                for heading in headings:
                    content_list=[]
                    next_element = heading.find_next_sibling()
                    while next_element and next_element.name not in ['h1']:
                        if next_element.name is not None:
                            content_list.append(next_element.get_text().strip())
                        next_element = next_element.find_next_sibling()
                    content_dict[heading.get_text().strip()]="\n".join(content_list)
                text=content_dict
            field_dict[title]=text
    field_dict["url"]=url
    return field_dict

def getFpoPatentInfo(fpo_search_result:FpoSearchResult):
    field_dict=getFpoPatentInfoByUrl(fpo_search_result.url)
    field_dict["patent_id"]=fpo_search_result.patent_id
    return dictToFpoPatentInfo(field_dict)

def downloadFpoPdfByUrl(pdf_url,save_path):
    res=requests.get(pdf_url)
    soup=BeautifulSoup(res.text, "html.parser")
    url_obj=soup.select_one("body > div > div > div:nth-child(3) > center:nth-child(10) > iframe")
    if url_obj:
        return downloadFile(url_obj["src"],save_path)
    else:
        raise Exception(f"无效的地址解析,{pdf_url}")

def dictToFpoPatentInfo(d,patent_id=None):
    if "patent_id" not in d:
        if patent_id is None:
            raise Exception("请指定patent_Id作为唯一标识")
        else:
            d["patent_id"]=patent_id
    return FpoPatentInfo(**d)

def downloadFpoPdf(fpo_patent_info:FpoPatentInfo,save_dir):
    if os.path.basename(save_dir).find(".pdf")!=-1:
        downloadFpoPdfByUrl(fpo_patent_info.pdf_url,save_dir)
    else:
        save_dir=os.path.join(save_dir,f"{fpo_patent_info.patent_id}.pdf")
        downloadFpoPdfByUrl(fpo_patent_info.pdf_url,save_dir)
def autoFpoSpider(query_txt,save_dir="data",save_pdf=True,num_in_search=1,id_is_query=True):
    fpo_search_result_list=getFpoSearchResult(query_txt=query_txt)
    createDirs(save_dir)
    for index,fpo_search_result in enumerate(fpo_search_result_list):
        if index+1>num_in_search:
            break
        fpo_patent_info=getFpoPatentInfo(fpo_search_result)
        base_patent_dir=os.path.join(save_dir,fpo_patent_info.patent_id)
        if id_is_query:
            base_patent_dir = os.path.join(save_dir, query_txt)
        createDirs(base_patent_dir)

        patent_info_dataframe,patent_content_dataframe=fpo_patent_info.toDataFrame()
        patent_info_file=os.path.join(base_patent_dir,f"info.xlsx")
        patent_info_dataframe.to_excel(patent_info_file,index=False)

        patent_content_file = os.path.join(base_patent_dir, f"content.xlsx")
        patent_content_dataframe.to_excel(patent_content_file,index=False)

        with open(os.path.join(base_patent_dir, f"origin_data.json"), "w", encoding="utf-8") as f:
            json.dump(fpo_patent_info.__dict__, f, ensure_ascii=False)
        if save_pdf:
            patent_pdf_file = os.path.join(base_patent_dir, f"patent.pdf")
            downloadFpoPdfByUrl(fpo_patent_info.pdf_url,patent_pdf_file)


import time
import re

import pandas as pd
from bs4 import BeautifulSoup
from urllib.parse import urlparse

from .utils import log

class GooglePatentInfo:
    def __init__(self, title='', abstract='', inventors='', application_number='', publication_number='',
                 publication_date='', pdf_url='', assignee_current='', claims_texts='', events=None,
                 classifications=None, descriptions='', patent_citations=None, non_patent_citations=None, city_bys=None,
                 similar_documents=None, legal_events=None, concepts=None):
        self.title = title
        self.abstract = abstract
        self.inventors = inventors
        self.application_number = application_number
        self.publication_number = publication_number
        self.publication_date = publication_date
        self.pdf_url = pdf_url
        self.assignee_current = assignee_current
        self.claims_texts = claims_texts
        self.events = events if events is not None else []
        self.classifications = classifications if classifications is not None else []
        self.descriptions = descriptions
        self.patent_citations = patent_citations if patent_citations is not None else []
        self.non_patent_citations = non_patent_citations if non_patent_citations is not None else []
        self.city_bys = city_bys if city_bys is not None else []
        self.similar_documents = similar_documents if similar_documents is not None else []
        self.legal_events = legal_events if legal_events is not None else []
        self.concepts = concepts if concepts is not None else []
    def toDataFrame(self):
        x={
            'title': [self.title],
            'abstract': [self.abstract],
            'inventors': [self.inventors],
            'application_number': [self.application_number],
            'publication_number': [self.publication_number],
            'publication_date': [self.publication_date],
            'pdf_url': [self.pdf_url],
            'assignee_current': [self.assignee_current],
            'claims_texts': [self.claims_texts],
            'events': [self.events],
            'classifications': [self.classifications],
            'patent_citations': [self.patent_citations],
            'non_patent_citations': [self.non_patent_citations],
            'city_bys': [self.city_bys],
            'similar_documents': [self.similar_documents],
            'legal_events': [self.legal_events],
            'concepts': [self.concepts]
        }
        x1={
        }
        for k,v in self.descriptions.items():
            x1[k]=[v]
        return pd.DataFrame(x),pd.DataFrame(x1)


def parseGooglePatentInfoHtml(html,url,engine="lxml"):
    parsed_url = urlparse(url)
    base_url=f"{parsed_url.scheme}://{parsed_url.netloc}"
    field_dict = {
        'title': '',
        'abstract': '',
        'inventors': '',
        'application_number': '',
        'publication_number':'',
        'publication_date': '',
        'pdf_url':'',
        'assignee_current':'',
        'claims_texts':'',
        'events':[],
        'classifications':[],
        'descriptions':'',
        'patent_citations':[],
        'non_patent_citations':[],
        'city_bys':[],
        'similar_documents':[],
        'legal_events':[],
        'concepts':[]
    }
    soup=BeautifulSoup(html,engine)
    title_span=soup.find("span",{"itemprop":"title"})
    if title_span is None:
        return None
    field_dict["title"] = title_span.get_text().strip()
    pdf_url_a=soup.find('a', {'itemprop': 'pdfLink'})
    if pdf_url_a:
        field_dict["pdf_url"] = pdf_url_a['href']
    abstract_el=soup.find(class_="abstract")
    if abstract_el is not None:
        field_dict["abstract"] = soup.find(class_="abstract").get_text().strip()
    field_dict["inventors"] = ', '.join([i.get_text().strip() for i in soup.findAll(itemprop="inventor")])
    field_dict["assignee_current"]=', '.join([i.get_text().strip() for i in soup.findAll(itemprop="assigneeCurrent")])
    field_dict["publication_number"] = soup.find(itemprop="publicationNumber").get_text().strip()
    field_dict["publication_date"]=soup.find(itemprop="publicationDate").get_text().strip()
    field_dict["application_number"]=soup.find(itemprop="applicationNumber").get_text().strip()
    events=soup.find_all(itemprop="events")
    if events:
        for event in events:
            event_document_id=event.find(itemprop="documentId")
            if event_document_id:
                event_document_id=event_document_id.get_text().strip()
            else:
                event_document_id=""
            event_dict={
                "date" : event.find(itemprop="date").get_text().strip(),
                "title" : event.find(itemprop="title").get_text().strip(),
                "type" : event.find(itemprop="type").get_text().strip(),
                "document_id" : event_document_id,
            }
            field_dict["events"].append(event_dict)
    classifications_list=soup.find_all("li",itemprop='classifications')
    if classifications_list:
        for classifications in classifications_list:
            classifications_dict={
                "code" : classifications.find(itemprop='Code').get_text().strip(),
                "description" : classifications.find(itemprop='Description').get_text().strip()
            }
            field_dict["classifications"].append(classifications_dict)
    claims_texts_div=soup.find(class_="claims")
    if claims_texts_div:
        field_dict["claims_texts"]= claims_texts_div.get_text().strip()
    description_div=soup.find(class_="description")
    if description_div is None:
        log(url+":\n"+str(soup),f"log/{time.time()}.log")
        raise Exception("无法解析正文:",url)
    field_dict["descriptions"] = description_div.get_text().strip()
    #提取
    patent_citations = soup.find_all("tr", {"itemprop": "backwardReferencesOrig"})
    if patent_citations is None or len(patent_citations)==0:
        patent_citations = soup.find_all("tr", {"itemprop": "backwardReferencesFamily"})
    for patent_citation in patent_citations:
        assignee_original_obj=patent_citation.find("span",{"itemprop":"assigneeOriginal"})
        patent_citation_dict={
            "publication_number":patent_citation.find("span",{"itemprop":"publicationNumber"}).get_text().strip(),
            "primary_language":patent_citation.find("span",{"itemprop":"primaryLanguage"}).get_text().strip(),
            "url":base_url+patent_citation.find("a")['href'],
            "priority_date":patent_citation.find("td",{"itemprop":"priorityDate"}).get_text().strip(),
            "publication_date":patent_citation.find("td",{"itemprop":"publicationDate"}).get_text().strip(),
            "assignee_original":"" if assignee_original_obj is None else assignee_original_obj.get_text().strip(),
            "title":patent_citation.find("td",{"itemprop":"title"}).get_text().strip(),
        }
        field_dict["patent_citations"].append(patent_citation_dict)
    non_patent_citations = soup.find_all("tr", {"itemprop": "detailedNonPatentLiterature"})
    if non_patent_citations:
        for non_patent_citation in non_patent_citations:
            # non_patent_citation.find("a")['href'],
            non_patent_citation_dict = {
                # "url": non_patent_citation.find("a")['href'],
                "title": non_patent_citation.find("span", {"itemprop": "title"}).get_text().strip(),
            }
            field_dict["non_patent_citations"].append(non_patent_citation_dict)
    city_bys = soup.find_all("tr", {"itemprop": "forwardReferencesFamily"})
    if city_bys is None or len(city_bys)==0:
        city_bys = soup.find_all("tr", {"itemprop": "forwardReferencesOrig"})
    for city_by in city_bys:
        city_by_original_obj=city_by.find("span", {"itemprop": "assigneeOriginal"})
        city_by_dict = {
            "publication_number": city_by.find("span", {"itemprop": "publicationNumber"}).get_text().strip(),
            "primary_language": city_by.find("span", {"itemprop": "primaryLanguage"}).get_text().strip(),
            "url": base_url + city_by.find("a")['href'],
            "priority_date": city_by.find("td", {"itemprop": "priorityDate"}).get_text().strip(),
            "publication_date": city_by.find("td", {"itemprop": "publicationDate"}).get_text().strip(),
            "assignee_original": ""if city_by_original_obj is None else city_by_original_obj.get_text().strip(),
            "title": city_by.find("td", {"itemprop": "title"}).get_text().strip(),
        }
        field_dict["city_bys"].append(city_by_dict)
    similar_documents = soup.find_all("tr", {"itemprop": "similarDocuments"})
    if similar_documents:
        for similar_document in similar_documents:
            publication_number=''
            scholar_id=''
            primary_language=''
            publication_date=''
            if similar_document.find("meta", {"itemprop": "isScholar"}):
                type="scholar"
                scholar_id=similar_document.find("meta", {"itemprop": "scholarID"})["content"]
            else:
                type="patent"
                primary_language_span=similar_document.find("span", {"itemprop": "primaryLanguage"})
                if primary_language_span:
                    primary_language=primary_language_span.get_text().strip()
                publication_number_span=similar_document.find("span",{"itemprop":"publicationNumber"})
                if publication_number_span:
                    publication_number=publication_number_span.get_text().strip()
            publication_date_time=similar_document.find("time", {"itemprop": "publicationDate"})
            if publication_date_time:
                publication_date=publication_date_time.get_text().strip()
            similar_document_dict = {
                "type":type,
                "scholar_id":scholar_id,
                "publication_number": publication_number,
                "primary_language": primary_language,
                "url": base_url + similar_document.find("a")['href'],
                "publication_date": publication_date,
                "title": similar_document.find("td", {"itemprop": "title"}).get_text().strip(),
            }
            field_dict["similar_documents"].append(similar_document_dict)
    legal_events = soup.find_all("tr", {"itemprop": "legalEvents"})

    if legal_events:
        for legal_event in legal_events:
            legal_event_dict = {
                "date": legal_event.find("time", {"itemprop": "date"}).get_text().strip(),
                "code": legal_event.find("td", {"itemprop": "code"}).get_text().strip(),
                "title": legal_event.find("td", {"itemprop": "title"}).get_text().strip(),
                "attributes": '\n'.join([i.get_text().strip().replace("\n","") for i in legal_event.findAll("p",itemprop="attributes")]),
            }
            field_dict["legal_events"].append(legal_event_dict)
    concepts_ul = soup.find("ul", {"itemprop": "concept"})
    if concepts_ul:
        for concept in concepts_ul.find_all("li", {"itemprop": "match"}):
            concept_dict = {
                "id": concept.find("span", {"itemprop": "id"}).get_text().strip(),
                "name":concept.find("span", {"itemprop": "name"}).get_text().strip(),
                "sections":', '.join([i.get_text().strip() for i in concept.find_all("span",itemprop="sections")]),
                "similarity":concept.find("span", {"itemprop": "similarity"}).get_text().strip(),
                "count":concept.find("span", {"itemprop": "count"}).get_text().strip(),
            }
            field_dict["concepts"].append(concept_dict)
    #保存正文原文
    return field_dict

RE_MAP={
    "cross_reference": [
        re.compile(r"CROSS[-\s]?REFERENCE(?: TO RELATED APPLICATIONS)?", re.IGNORECASE)
    ],
    "related_applications": [
        re.compile(r"RELATED APPLICATIONS", re.IGNORECASE)
    ],
    "technical_field": [
        re.compile(r"Technical Field|技术领域|FIELD OF (?:INVENTION|PRESENT APPLICATION)|TECHNICAL FIELD|FIELD", re.IGNORECASE)
    ],
    "background": [
        re.compile(r"Background|背景技术|BACKGROUND OF (?:THE INVENTION|INVENTION|PRESENT APPLICATION)|BACKGROUND INFORMATION", re.IGNORECASE)
    ],
    "disclosure_of_invention": [
        re.compile(r"Disclosure of Invention|发明内容|SUMMARY OF (?:THE INVENTION|INVENTION)|BRIEF SUMMARY OF THE PRESENT APPLICATION", re.IGNORECASE)
    ],
    "drawings": [
        re.compile(r"Drawings|附图说明|BRIEF DESCRIPTION OF (?:THE DRAWINGS|DRAWINGS)", re.IGNORECASE)
    ],
    "detailed_description": [
        re.compile(r"Detailed Description|具体实施方式|DETAILED DESCRIPTION OF (?:THE INVENTION|INVENTION)|DESCRIPTION OF THE PREFERRED EMBODIMENTS", re.IGNORECASE)
    ]
}
def parseGooglePatentDescription(text, map=None):
    # 定义默认的映射字典
    if map is None:
        map = RE_MAP
    # 使用字典保存结果
    sections = {key: "" for key in map}

    # 按行分割文本并清理空白字符
    lines = text.strip().split("\n")
    current_header = None

    # 遍历每一行，提取对应的文本
    for line in lines:
        line = line.strip()
        if not line:  # 跳过空行
            continue
        # 查找当前行的标题
        for header, patterns in map.items():
            if any(pattern.search(line) for pattern in patterns):
                current_header = header
                break
        if current_header:
            sections[current_header] += line + "\n"
    # 去除每个部分末尾的换行符
    return {key: value.strip() for key, value in sections.items()}


import os

import wget
from tqdm import tqdm

def log(msg,file_name):
    if not os.path.exists(os.path.basename(file_name)):
        createDirs(os.path.dirname(file_name))
    with open(file_name, 'a',encoding="utf-8") as f:
        f.write(str(msg)+'\n')
def downloadFile(file_url, save_path):
    def bar_custom(current, total, width=80):
        t.total = total
        t.update(current)
    if os.path.exists(save_path):
        os.remove(save_path)
    createDirs(os.path.dirname(save_path))
    with tqdm(unit='', unit_scale=True, unit_divisor=1024, miniters=1, desc="Downloading") as t:
        return wget.download(file_url, save_path, bar=bar_custom)

def createDirs(path):
    if not os.path.exists(path):
        os.makedirs(path)
        return True
    return False
